/* db.c */

/* Jadrova cast databaze. */

#include <malloc.h>
#include <stdio.h>

#include "kernel.h"



/*
 * receive all info from database
 *
 * it expects kernel locked for writing
 */
int db_recv(int sock) {
  message_t *message;
  uint16_t sensorno;
  uint16_t user_count, user_number, tz_count, tz_number;
  int i;

/* get number and addreses of sensors */
  /* ask */
  llprintf(TRACE_MSG, "db: sending MSG_DB_REQUEST_SENSORS...");
  if (!kernel_send_simple_message(sock, MSG_DB_REQUEST_SENSORS)) return -1;
  llprintf(TRACE_MSG, "OK\n");
  /* response */
  llprintf(TRACE_MSG, "db: waiting for MSG_DB_SENSORS\n");
  if ((message = kernel_recv(sock, TIMEOUT, -1)) == NULL) return -1;
#if TRACE_MSG
  message_dump("db: prijato MSG_DB_SENSORS, ", message);
#endif
  if (message->op != MSG_DB_SENSORS) {
    printf("db: Ocekava se uvodni ramec s informacemi o senzorech\n");
    goto exception_free;
  }
  llprintf(TRACE_MSG, "OK\n");
  /* parse */
  if (!message_parse_part(message, 0, "B", &nsensors)) goto exception_free;
  lprintf("db: pocet senzoru: %d\n", nsensors);
  if (nsensors > MAX_SENSORS) { /* nemozne */
    nsensors = 0;
    printf("db: Prilis mnoho senzoru, ocekavano max. %d\n", MAX_SENSORS);
    goto exception_free;
  }
  for (i = 0; i < nsensors; i++) {
    int id;
    
    if (!message_parse_part(message, 1 + 4 * i, "WW", &id, &snimace[i].room)) goto exception_free;
    snimace[i].id = id;
    snimace[i].users = NULL;
    snimace[i].tz = NULL;
    snimace[i].nusers = 0;
    snimace[i].nzones = 0;
    snimace[i].active = -1;
  }
  message_free(message);

  /* get all user info */
  /* ask */
  llprintf(TRACE_MSG, "db: sending MSG_DB_REQ_ALL_USERS...");
  if (!kernel_send_simple_message(sock, MSG_DB_REQ_ALL_USERS)) return -1;
  llprintf(TRACE_MSG, "OK\n");
  /* response (header) */
  if ((message = kernel_recv(sock, TIMEOUT, -1)) == NULL) return -1;
#if TRACE_MSG
  message_dump("db: prijato, ", message);
#endif
  if (!message_test(message, "db", MSG_DB_HEADER, "W")) return 0;
  if (!message_parse(message, "W", &nusers)) return 0;
  message_free(message);
  lprintf("db: %d user info\n", nusers);
  /* all responses */
  if (nusers) {
    uzivatele = malloc(nusers * sizeof(user_info_t));
    for (i = 0; i < nusers; i++) {
      llprintf(TRACE_MSG, "db: prijem MSG_DB_ALL_USERS...");
      if ((message = kernel_recv(sock, TIMEOUT, -1)) == NULL) return -1;
      if (!message_test(message, "db", MSG_DB_ALL_USERS, "IKKKKWW")) {
        free(message);
        return 0;
      }
      if (!message_parse(message, "IKKKKWW", uzivatele[i].id_jis, uzivatele[i].key, uzivatele[i].jmeno, uzivatele[i].prijmeni, uzivatele[i].os_cislo, &uzivatele[i].id_user, &uzivatele[i].id_group)) {
        printf("db: asi je spatne pocet informaci o userech (aktualni: %d, pocet: %d)\n)",i, nusers);
        free(message);
        return 0;
      }
      free(message);
      llprintf(TRACE_MSG, "db: OK\n");
      uzivatele[i].prihlasen = 0;
      uzivatele[i].addr_snimace = 0;
    }
  } else uzivatele = NULL;

  /* get tables of each sensor */
  for (sensorno = 0; sensorno < nsensors; sensorno++) {
  /* ---- list of users --- */
    llprintf(TRACE_MSG, "db: sending MSG_DB_REQUEST_USERS and waiting for header...");
    switch (kernel_sendreceive_header(sock, "db", MSG_DB_REQUEST_USERS, snimace[sensorno].id, &user_number)) {
      case -1: return -1;
      case 0: printf("db: mozna neodpovida pocet informaci o userech\n"); return 0;
      default: break;
    }
    llprintf(TRACE_MSG, "OK (%d users)\n", user_number);
    if (user_number) snimace[sensorno].users = malloc(user_number * sizeof(user_t));
    else snimace[sensorno].users = NULL;
    for (user_count = 0; user_count < user_number; user_count++) {
      user_t *user = snimace[sensorno].users + user_count;
      int id_zone, rights;
      jis_t id;
      
      /* receive data about users */
      if ((message = kernel_recv(sock, TIMEOUT, -1)) == NULL) return -1;
      if (message->op != MSG_DB_USER) {
        printf("db: Ocekává se informace o uzivatelich (typ %02X, prijato %02X)\n", MSG_DB_USER, message->op);
        printf("db: asi je spatne pocet useru (aktualni: %d, pocet: %d)\n)", user_count, user_number);
        goto exception_free_all;
      }
#if TRACE_MSG
      message_dump("db: prijato MSG_DB_USER, ", message);
#endif
      /* get the data */
      if (!message_parse(message, "IBW", id, &id_zone, &rights))
	goto exception_free_all;
      user->id_zone = id_zone;
      user->rights = rights;
      if ((user->info = kernel_search_user(id)) == NULL) {
	printf("db: uzivatel s id 0x%02X%02X%02X%02X%02X%02X%02X%02X nenalezen\n", id[0], id[1], id[2], id[3], id[4], id[5], id[6], id[7]);
	goto exception_free_all;
      }

      message_free(message);
    }
    snimace[sensorno].nusers = user_count;

  /* ---- list of time zones --- */
    llprintf(TRACE_MSG, "db: sending MSG_DB_REQUEST_TIME_ZONE and waiting for header...");
    switch (kernel_sendreceive_header(sock, "db", MSG_DB_REQUEST_TIME_ZONE, snimace[sensorno].id, &tz_number)) {
      case -1: return -1;
      case 0: printf("db: mozna neodpovida pocet useru\n"); return 0;
      default: break;
    }
    llprintf(TRACE_MSG, "OK (%d time zones)\n", tz_number);
    if (tz_number) snimace[sensorno].tz = malloc(tz_number * sizeof(time_zone_t));
    else snimace[sensorno].tz = NULL;
    for (tz_count = 0; tz_count < tz_number; tz_count++) {
      time_zone_t *tz = snimace[sensorno].tz + tz_count;
      int id_zone;
      
      /* receive data about time zones */
      if ((message = kernel_recv(sock, TIMEOUT, -1)) == NULL) return -1;
      if (message->op != MSG_DB_TIME_ZONE) {
        printf("db: Ocekává se informace o casovych zonach (typ %02X, prijato %02X)\n", MSG_DB_TIME_ZONE, message->op);
        printf("db: asi je spatne pocet zon (aktualni: %d, pocet: %d)\n)", tz_count, tz_number);
        goto exception_free_all;
      }
#if TRACE_MSG
      message_dump("db: prijato MSG_DB_TIME_ZONE, ", message);
#endif
      /* get the data */
      if (!message_parse(message, "BT", &id_zone, tz->bmp)) goto exception_free_all;
      tz->id = id_zone;

      message_free(message);
    }
    snimace[sensorno].nzones = tz_number;
  }

  return 1;

exception_free_all:
  kernel_dbfree();
exception_free:
  message_free(message);
  return 0;
}



