/***************************************************
 *                MODUL jadro-sql                  *
 * modul komunikuje s modulem jadro a SQL databazi *
 *                                                 *
 *             (C) Michal Petrovic                 *
 ***************************************************/  
 
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>
#include <pthread.h>

#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "kernel.h"
#include "def.h"

#include <mysql/mysql.h>


#define   DB_SERVER	"localhost"
#define   DB_USER	"ds2003a"
#define   DB_PASS	".ds.2003a."
/*
#define   DB_SERVER	"dima"
#define   DB_USER	"ds"
#define   DB_PASS	"pivo"
*/
#define   DB_NAME	"ds2003a"
#define   TIME_WAIT 10

pthread_mutex_t zamek;         /* ochrana globalnich dat */
int zmena_db;

/*********************************************/
/*    zamkne zamek pro sdilenou promenou     */
/*********************************************/
static int zamkni() {
  return pthread_mutex_lock(&zamek) == 0;
}

/*********************************************/
/*    odemkne zamek pro sdilenou promenou    */
/*********************************************/
static int odemkni() {
  return pthread_mutex_unlock(&zamek) == 0;
}

/*********************************************/
/*      vrati cislo aktualniho mesice        */
/*********************************************/
int vrat_mesic() {
  struct tm tm_time;
  time_t times;

  time(&times);
  tm_time = *localtime(&times);
  return tm_time.tm_mon+1;
}

/*********************************************/
/*      vrati cislo aktualniho tydne         */
/*********************************************/
int vrat_tyden() {
  struct tm tm_time;
  time_t times;
  int ret = 0;

  time(&times);
  tm_time = *localtime(&times);
  if ((0  < tm_time.tm_mday)&&(tm_time.tm_mday < 8)) ret = 1;
  if ((7  < tm_time.tm_mday)&&(tm_time.tm_mday < 15)) ret = 2;
  if ((14 < tm_time.tm_mday)&&(tm_time.tm_mday < 22)) ret = 3;
  if ((21 < tm_time.tm_mday)&&(tm_time.tm_mday < 32)) ret = 4;
  return ret;
}

/***********************************************************/
/* connect to remote host to given port, return the socket */
/***********************************************************/
int module_connect(const char *host, uint16_t port) {
  int sock;
  struct sockaddr_in addr;
  struct hostent *h_info;

  /* create socket */
  if ((sock = socket(AF_INET, SOCK_STREAM, 0))  == -1) {
    printf("Nelze inicializovat soket: %s\n", strerror(errno));
    return -1;
  }
  /* set remote address of kernel */
  if ((h_info = gethostbyname(host)) == NULL) {
    printf("Chyba zjistovani \"%s\": %s\n", host, hstrerror(h_errno));
    goto exception0;
  }
  memset(&addr, 0, sizeof(addr));
  addr.sin_family = AF_INET;
  addr.sin_port = htons(port);
  memcpy(&addr.sin_addr, h_info->h_addr, h_info->h_length);
  /* connect to the kernel */
  if (connect(sock, (struct sockaddr *)&addr, sizeof(addr)) == -1) {
    printf("Chyba pri pripojovani: %s\n", strerror(errno));
    goto exception0;
  }
  return sock;
  exception0:
  close(sock);
  return -1;
}

/*********************************************/
/*      obsluha a komunikace s jadrem        */
/*********************************************/
void run(int sock) {
  MYSQL my_connection;
  MYSQL_RES *result;
  MYSQL_ROW row;
  unsigned int num_rows;
  char sql_prikaz[1000];
  uint16_t id_sensor;
  uint16_t id_mistnosti;
  uint16_t id_skupina;
  uint16_t id_user;
  uint16_t prava;
  uint8_t id_zony;
  uint8_t id_jis[8];
  uint8_t key[16];
  uint8_t bmp[BITMAP_SIZE];
  char text[3] = {0,0,0};
  char jmeno[16];
  char prijmeni[16];
  char os_cislo[16];
  char str_log[STR_MAXLEN]; 
  message_t *msg;
  int i, j, k;
  static const char *mesic[12]={"leden", "unor", "brezen", "duben", "kveten",
                                "cerven", "cervenec", "srpen", "zari", "rijen",
                                "listopad", "prosinec"};
 
  printf("Pripojuji se k modulu jadro..."); 
  msg = kernel_recv(sock, 1000, 1);
  if(msg == NULL) {
    fprintf(stdout, "Chyba pri prijmu od kernel: MSG_DB_INIT (%02X).\n", MSG_DB_INIT);
    return;
  }
  printf("OK\nCekam na inicializaci od jadra..."); 
  if(msg->op != MSG_DB_INIT) {
    printf("Zprava neni MSG_DB_INIT (prijato 0x%02X)\n", MSG_DB_INIT);
    return;
  }
  printf("OK\n");
  message_free(msg);
  while (1) {
    /**************************************************/
    /* poslu zpravu DB_RENEW pac se neco zmenilo v DB */
    /**************************************************/
    if (zmena_db){
      printf("cekani na zamek...\n");
      zamkni();
      zmena_db = 0;
      odemkni();
      kernel_send_simple_message(sock, MSG_DB_RENEW);
      printf("MSG_DB_RENEW: RENEW zaregistrovano a poslano jadru.\n");
    }
    
    /***********************************/
    /*   cekam na zpravu od jadra...   */
    /***********************************/
    msg = kernel_recv(sock, 4000, 1);
    if(msg == NULL) {
      printf("Chyba pri prijmu od kernel\n");
      return;
    }
    #if TRACE_MSG
      message_dump("\nPrijato od jadra:", msg);
    #endif
    switch (msg->op) {
      /***********************************/
      /*      jadro zada o cidla         */
      /***********************************/
      case MSG_DB_REQUEST_SENSORS:
       /* pripojeni do DB + SELECT */     
        mysql_init(&my_connection); 
        if (!mysql_real_connect(&my_connection, DB_SERVER, DB_USER, DB_PASS, DB_NAME, 0, NULL, 0)) {
          fprintf(stderr, "Nepodarilo se pripojit k DB!\n");
          kernel_send_simple_message(sock, MSG_DB_FAIL);
          if (mysql_errno(&my_connection)) {
            fprintf(stderr, "Connection error %d: %s\n",
            mysql_errno(&my_connection), mysql_error(&my_connection));
          }
        } else { 
          /* vytvoreni SQl dotazu a provedeni */
          if ( mysql_query(&my_connection, "SELECT snimac,id_mistnosti FROM snimac") ) {
            printf("MSG_DB_SENSORS: SELECT error: %s\n", mysql_error(&my_connection));
            kernel_send_simple_message(sock, MSG_DB_FAIL);
          } else {
            result = mysql_store_result(&my_connection);
            /* zjisteni poctu uzivatelu pro dany sensor */
            num_rows = mysql_num_rows(result);
            printf("MSG_DB_SENSORS: Pocet cidel:%d\n",num_rows); fflush(stdout);
            i = 0;
            /* posilani cidel */
            msg = message_create_part(MSG_DB_SENSORS, num_rows*4, "B", num_rows);
            while ((row = mysql_fetch_row(result))) {
              sscanf(row[0],"%04hx",&id_sensor);
              sscanf(row[1],"%04hx",&id_mistnosti);
              #if TRACE_MSG
                printf("MSG_DB_SENSORS: id_sensor[%04hx] id_mistnosti[%02hx]\n",id_sensor,id_mistnosti); fflush(stdout);
              #endif
              printf("MSG_DB_SENSORS: Posilam %d. cidlo[%04hx]\n",i+1,id_sensor); fflush(stdout);
              message_add_part(msg, 1 + 2 * i, "WW", id_sensor,id_mistnosti);
              i++;
            }
            kernel_send(sock, msg);
            message_free(msg);
            printf("MSG_DB_SENSORS: Konec posilani cidel\n"); fflush(stdout);
            mysql_free_result(result);
          }
          mysql_close(&my_connection);
        }
        break;

      /***********************************/
      /*        jadro zada o usery       */
      /***********************************/
      case MSG_DB_REQUEST_USERS:
        if (!message_parse(msg, "W", &id_sensor)) return;
        message_free(msg);
        /* pripojeni do DB + SELECT */
        mysql_init(&my_connection); 
        if (!mysql_real_connect(&my_connection, DB_SERVER, DB_USER, DB_PASS, DB_NAME, 0, NULL, 0)) {
          fprintf(stderr, "Nepodarilo se pripojit k DB!\n");
          kernel_send_simple_message(sock, MSG_DB_FAIL);
          if (mysql_errno(&my_connection)) {
            fprintf(stderr, "Connection error %d: %s\n",
            mysql_errno(&my_connection), mysql_error(&my_connection));
          }
        } else { 
          /* vytvoreni SQl dotazu a provedeni */
          sprintf(sql_prikaz, "SELECT  id_jis,id_cas_zona,prava FROM snimac, mistnosti, prava, skupina, uzivatel, mesic, tyden,cas_zona WHERE snimac.id_mistnosti = mistnosti.id_mistnosti AND mistnosti.id_mistnosti = prava.id_mistnosti AND prava.id_prava = skupina.id_prava AND skupina.id_skupina = uzivatel.id_skupina AND prava.id_mesic = mesic.id_mesic AND mesic.%s = tyden.id_tyden AND tyden.tyden%d = cas_zona.id_cas_zona AND snimac.snimac = '%04x'", mesic[vrat_mesic()],vrat_tyden(), id_sensor);
          if ( mysql_query(&my_connection, sql_prikaz) ) {
            printf("MSG_DB_USER: SELECT error: %s\n", mysql_error(&my_connection));
            kernel_send_simple_message(sock, MSG_DB_FAIL);
          } else {
            result = mysql_store_result(&my_connection);
            /* zjisteni poctu uzivatelu pro dany sensor */
            num_rows = mysql_num_rows(result);
            printf("MSG_DB_USER: Pocet uzivatelu:%d\n",num_rows); fflush(stdout);
             /* hlavicka pro uzivatele */
            printf("MSG_DB_USER: id_sensor[%04hx] posilam hlavicku usera...",id_sensor); fflush(stdout);
            msg = message_create(MSG_DB_HEADER, "W", num_rows);
            kernel_send(sock, msg);
            message_free(msg);
            printf("OK\n"); fflush(stdout);
            i = 1;
            /* posilani jednotlivych useru */
            while ((row = mysql_fetch_row(result))&&(num_rows>0)) {
              for (k=0;k<16;k+=2){
	            text[0]=row[0][k];
	            text[1]=row[0][k+1];
	            sscanf(text,"%hhx",&id_jis[k/2]);
	          }
              sscanf(row[1],"%hhx",&id_zony);
              sscanf(row[2],"%04hx",&prava);
              #if TRACE_MSG
                printf("MSG_DB_USER: id_jis["); fflush(stdout);
                for (j = 0; j < 8; j++) printf(" %02hx", id_jis[j]);
                printf("] id_zony[%02hhx] prava[%02hx]\n", id_zony, prava); fflush(stdout);
              #endif
              printf("MSG_DB_USER: id_sensor[%04hx] posilam %d.usera.",id_sensor, i); fflush(stdout);
              msg = message_create(MSG_DB_USER, "IBW", id_jis, id_zony, prava);
              kernel_send(sock, msg);
              message_free(msg);
              printf("OK\n"); fflush(stdout);
              i++;
            }
            printf("MSG_DB_USER: Konec posilani useru.\n"); fflush(stdout);
            mysql_free_result(result);
          }       
          mysql_close(&my_connection);
        }
        break;

      /***********************************/
      /*     jadro zada o casove zony    */
      /***********************************/
      case MSG_DB_REQUEST_TIME_ZONE:
        if (!message_parse(msg, "W", &id_sensor)) return;
        message_free(msg);
        /* pripojeni do DB + SELECT */
        mysql_init(&my_connection); 
        if (!mysql_real_connect(&my_connection, DB_SERVER, DB_USER, DB_PASS, DB_NAME, 0, NULL, 0)) {
          fprintf(stderr, "Nepodarilo se pripojit k DB!\n");
          kernel_send_simple_message(sock, MSG_DB_FAIL);
          if (mysql_errno(&my_connection)) {
            fprintf(stderr, "Connection error %d: %s\n",
            mysql_errno(&my_connection), mysql_error(&my_connection));
          }
        } else { 
          /* vytvoreni SQl dotazu a provedeni */
          sprintf(sql_prikaz, "SELECT  id_cas_zona, maska FROM snimac, mistnosti, prava, mesic, tyden, cas_zona WHERE snimac.id_mistnosti = mistnosti.id_mistnosti AND mistnosti.id_mistnosti = prava.id_mistnosti AND prava.id_mesic = mesic.id_mesic AND mesic.%s = tyden.id_tyden AND snimac.snimac = '%04x' GROUP BY id_cas_zona", mesic[vrat_mesic()], id_sensor);
          if ( mysql_query(&my_connection, sql_prikaz) ) {
            printf("MSG_DB_TIME_ZONE: SELECT error: %s\n", mysql_error(&my_connection));
            kernel_send_simple_message(sock, MSG_DB_FAIL);
          } else {
            result = mysql_store_result(&my_connection);
            /* zjisteni poctu casovych zon pro dany sensor */
            num_rows = mysql_num_rows(result);
            printf("MSG_DB_TIME_ZONE: Pocet casovych zon:%d\n",num_rows); fflush(stdout);
            /* hlavicka pro casovy zony */
            printf("MSG_DB_TIME_ZONE: id_sensor[%04x] posilam hlavicku TZ...",id_sensor); fflush(stdout);
            msg = message_create(MSG_DB_HEADER, "W", num_rows);
            kernel_send(sock, msg);
            message_free(msg);
            printf("OK\n"); fflush(stdout); 
            i = 1;
            /* posilani jednotlivych casovych zon */
            while ((row = mysql_fetch_row(result))) {
               sscanf(row[0],"%hhx",&id_zony);
               for (j = 0; j < BITMAP_SIZE; j++) {
                 bmp[j] = 0;
                 for (k = 0; k < 8; k++) {
                   bmp[j] *= 2;
                   if (row[1][j*8+k] == '1') bmp[j]++;
                 }
               }
               #if TRACE_MSG
                 printf("MSG_DB_TIME_ZONE: id_zony[%02hx] bmp[",id_zony); fflush(stdout);
                 for (j = 0; j < BITMAP_SIZE; j++) printf(" %02hx",bmp[j]);
                 printf("]\n"); fflush(stdout);
               #endif
               printf("MSG_DB_TIME_ZONE: id_sensor[%04hx] posilam %d.TZ...", id_sensor, i); fflush(stdout);
               msg = message_create(MSG_DB_TIME_ZONE, "BT", id_zony, bmp);
               kernel_send(sock, msg);
               message_free(msg);
               printf("OK\n"); fflush(stdout);
               i++;
            }
            printf("MSG_DB_TIME_ZONE: Konec posilani casovych zon.\n"); fflush(stdout);
            mysql_free_result(result);
          }       
          mysql_close(&my_connection);
        }
        break;

      /***********************************/
      /*   jadro zada informace o useru  */
      /***********************************/
      case MSG_DB_REQ_ALL_USERS:
        /* pripojeni do DB + SELECT */
        mysql_init(&my_connection); 
        if (!mysql_real_connect(&my_connection, DB_SERVER, DB_USER, DB_PASS, DB_NAME, 0, NULL, 0)) {
          fprintf(stderr, "Nepodarilo se pripojit k DB!\n");
          kernel_send_simple_message(sock, MSG_DB_FAIL);
          if (mysql_errno(&my_connection)) {
            fprintf(stderr, "Connection error %d: %s\n",
            mysql_errno(&my_connection), mysql_error(&my_connection));
          }
        } else { 
          /* vytvoreni SQl dotazu a provedeni */
          sprintf(sql_prikaz, "SELECT  id_jis,jis_key,id_uzivatel,skupina.id_skupina,jmeno,prijmeni,os_cislo FROM snimac, mistnosti, prava, skupina, uzivatel, mesic, tyden,cas_zona WHERE snimac.id_mistnosti = mistnosti.id_mistnosti AND mistnosti.id_mistnosti = prava.id_mistnosti AND prava.id_prava = skupina.id_prava AND skupina.id_skupina = uzivatel.id_skupina AND prava.id_mesic = mesic.id_mesic AND mesic.%s = tyden.id_tyden AND tyden.tyden%d = cas_zona.id_cas_zona AND snimac.snimac = '%04x'", mesic[vrat_mesic()],vrat_tyden(), id_sensor);
          if ( mysql_query(&my_connection, sql_prikaz) ) {
            printf("MSG_DB_ALL_USERS: SELECT error: %s\n", mysql_error(&my_connection));
            kernel_send_simple_message(sock, MSG_DB_FAIL);
          } else {
            result = mysql_store_result(&my_connection);
            /* zjisteni poctu uzivatelu pro dany sensor */
            num_rows = mysql_num_rows(result);
            printf("MSG_DB_ALL_USERS: Pocet uzivatelu:%d\n",num_rows); fflush(stdout);
             /* hlavicka pro uzivatele */
            printf("MSG_DB_ALL_USERS: id_sensor[%04hx] posilam hlavicku usera...",id_sensor); fflush(stdout);
            message_free(msg);
            msg = message_create(MSG_DB_HEADER, "W", num_rows);
            kernel_send(sock, msg);
            message_free(msg);
            printf("OK\n"); fflush(stdout);
            i = 1;
            /* posilani jednotlivych useru */
            while ((row = mysql_fetch_row(result))&&(num_rows>0)) {
              for (k=0;k<16;k+=2){
	        text[0]=row[0][k];
	        text[1]=row[0][k+1];
	        sscanf(text,"%hhx",&id_jis[k/2]);
	      }
              for (k=0;k<32;k+=2){
	        text[0]=row[1][k];
	        text[1]=row[1][k+1];
	        sscanf(text,"%hhx",&key[k/2]);
	      }
               sscanf(row[2],"%04hx",&id_user);
               sscanf(row[3],"%04hx",&id_skupina);
               strncpy(jmeno, row[4], 16);
               strncpy(prijmeni, row[5], 16);
               strncpy(os_cislo, row[6], 16);
               #if TRACE_MSG
                 printf("MSG_DB_ALL_USERS: id_jis[");fflush(stdout);
                 for (j = 0; j < 8; j++) printf(" %02hx",id_jis[j]);
                 printf("] key[");
                 for (j = 0; j < 16; j++) printf(" %02hx",key[j]);
                 printf("] jmeno[%s] prijmeni[%s] os_cislo[%s] id_user[%02hx] id_skupina[%02hx]\n", jmeno, prijmeni, os_cislo, id_user, id_skupina); fflush(stdout);
               #endif
               msg = message_create(MSG_DB_ALL_USERS, "IKKKKWW", id_jis, key, jmeno,prijmeni,os_cislo,id_user,id_skupina);
               printf("MSG_DB_ALL_USERS: id_sensor[%04hx] posilam %d.usera ", id_sensor, i); fflush(stdout);
               if (!kernel_send(sock, msg)) return;
               message_free(msg);
               printf("OK\n"); fflush(stdout);
               i++;
            }
            printf("MSG_DB_ALL_USERS: Konec posilani useru.\n"); fflush(stdout);
            mysql_free_result(result);
          }       
          mysql_close(&my_connection);
        }
        break;

      /***********************************/
      /*  jadro zada informace o 1 useru */
      /***********************************/
      case MSG_DB_REQ_ONE_USER:
        if (!message_parse(msg, "WI", &id_sensor, &id_user)) return;
        message_free(msg);
        /* pripojeni do DB + SELECT */
        mysql_init(&my_connection); 
        if (!mysql_real_connect(&my_connection, DB_SERVER, DB_USER, DB_PASS, DB_NAME, 0, NULL, 0)) {
          fprintf(stderr, "Nepodarilo se pripojit k DB!\n");
          kernel_send_simple_message(sock, MSG_DB_FAIL);
          if (mysql_errno(&my_connection)) {
            fprintf(stderr, "Connection error %d: %s\n",
            mysql_errno(&my_connection), mysql_error(&my_connection));
          }
        } else { 
          /* vytvoreni SQl dotazu a provedeni */
          sprintf(sql_prikaz, "SELECT  id_jis,id_cas_zona,prava FROM snimac, mistnosti, prava, skupina, uzivatel, mesic, tyden,cas_zona WHERE snimac.id_mistnosti = mistnosti.id_mistnosti AND mistnosti.id_mistnosti = prava.id_mistnosti AND prava.id_prava = skupina.id_prava AND skupina.id_skupina = uzivatel.id_skupina AND prava.id_mesic = mesic.id_mesic AND mesic.%s = tyden.id_tyden AND tyden.tyden%d = cas_zona.id_cas_zona AND uzivatel.id_uzivatel = '%d'", mesic[vrat_mesic()],vrat_tyden(), id_user);
          if ( mysql_query(&my_connection, sql_prikaz) ) {
            printf("MSG_DB_REQ_ONE_USER: SELECT error: %s\n", mysql_error(&my_connection));
            kernel_send_simple_message(sock, MSG_DB_FAIL);
          } else {
            result = mysql_store_result(&my_connection);
            /* zjisteni poctu uzivatelu pro dany sensor */
            num_rows = mysql_num_rows(result);
            printf("MSG_DB_REQ_ONE_USER: Pocet uzivatelu:%d\n",num_rows); fflush(stdout);
             /* hlavicka pro uzivatele */
            printf("MSG_DB_REQ_ONE_USER: id_sensor[%04hx] posilam hlavicku usera...",id_sensor); fflush(stdout);
            msg = message_create(MSG_DB_HEADER, "W", num_rows);
            kernel_send(sock, msg);
            message_free(msg);
            printf("OK\n"); fflush(stdout);
            i = 1;
            /* posilani jednotlivych useru */
            while ((row = mysql_fetch_row(result))&&(num_rows>0)) {
              for (k=0;k<16;k+=2){
	            text[0]=row[0][k];
	            text[1]=row[0][k+1];
	            sscanf(text,"%hhx",&id_jis[k/2]);
	          }
              sscanf(row[1],"%hhx",&id_zony);
              sscanf(row[2],"%04hx",&prava);
              #if TRACE_MSG
                printf("MSG_DB_REQ_ONE_USER: id_jis["); fflush(stdout);
                for (j = 0; j < 8; j++) printf(" %02hx", id_jis[j]);
                printf("] id_zony[%02hhx] prava[%02hx]\n", id_zony, prava); fflush(stdout);
              #endif
              printf("MSG_DB_REQ_ONE_USER: id_sensor[%04hx] posilam %d.usera.",id_sensor, i); fflush(stdout);
              msg = message_create(MSG_DB_USER, "IBW", id_jis, id_zony, prava);
              kernel_send(sock, msg);
              message_free(msg);
              printf("OK\n"); fflush(stdout);
              i++;
            }
            printf("MSG_DB_REQ_ONE_USER: Konec posilani useru.\n"); fflush(stdout);
            mysql_free_result(result);
          }       
          mysql_close(&my_connection);
        }
        break;

      /*************************************/
      /* jadro zada informace o 1 cas.zonu */
      /*************************************/
      case MSG_DB_REQ_ONE_TIME_ZONE:
        if (!message_parse(msg, "WB", &id_sensor, &id_zony)) return;
        message_free(msg);
        /* pripojeni do DB */     
        mysql_init(&my_connection); 
        if (!mysql_real_connect(&my_connection, DB_SERVER, DB_USER, DB_PASS, DB_NAME, 0, NULL, 0)) {
          fprintf(stderr, "Nepodarilo se pripojit k DB!\n");
          kernel_send_simple_message(sock, MSG_DB_FAIL);
          if (mysql_errno(&my_connection)) {
            fprintf(stderr, "Connection error %d: %s\n",
            mysql_errno(&my_connection), mysql_error(&my_connection));
          }
        } else { 
          /* vytvoreni SQl dotazu a provedeni */
          sprintf(sql_prikaz, "SELECT  id_cas_zona, maska FROM cas_zona WHERE cas_zona.id_zona='%d'", id_zony); /* pozor id_zony je 1B tzn. max 256 zon:( */
          if ( mysql_query(&my_connection, sql_prikaz) ) {
            printf("MSG_DB_REQ_ONE_TIME_ZONE: SELECT error: %s\n", mysql_error(&my_connection));
            kernel_send_simple_message(sock, MSG_DB_FAIL);
          } else {
            result = mysql_store_result(&my_connection);
            /* zjisteni poctu casovych zon pro dany sensor */
            num_rows = mysql_num_rows(result);
            printf("MSG_DB_REQ_ONE_TIME_ZONE: Pocet casovych zon:%d\n",num_rows); fflush(stdout);
            /* hlavicka pro casovy zony */
            printf("MSG_DB_REQ_ONE_TIME_ZONE: id_sensor[%04x] posilam hlavicku TZ...",id_sensor); fflush(stdout);
            msg = message_create(MSG_DB_HEADER, "W", num_rows);
            kernel_send(sock, msg);
            message_free(msg);
            printf("OK\n"); fflush(stdout); 
            i = 1;
            /* posilani jednotlivych casovych zon */
            while ((row = mysql_fetch_row(result))) {
               sscanf(row[0],"%hhx",&id_zony);
               for (j = 0; j < BITMAP_SIZE; j++) {
                 bmp[j] = 0;
                 for (k = 0; k < 8; k++) {
                   bmp[j] *= 2;
                   if (row[1][j*8+k] == '1') bmp[j]++;
                 }
               }
               #if TRACE_MSG
                 printf("MSG_DB_REQ_ONE_TIME_ZONE: id_zony[%02hx] bmp[",id_zony); fflush(stdout);
                 for (j = 0; j < BITMAP_SIZE; j++) printf(" %02hx",bmp[j]);
                 printf("]\n"); fflush(stdout);
               #endif
               printf("MSG_DB_REQ_ONE_TIME_ZONE: id_sensor[%04hx] posilam %d.TZ...", id_sensor, i); fflush(stdout);
               msg = message_create(MSG_DB_TIME_ZONE, "BT", id_zony, bmp);
               kernel_send(sock, msg);
               message_free(msg);
               printf("OK\n"); fflush(stdout);
               i++;
            }
            printf("MSG_DB_REQ_ONE_TIME_ZONE: Konec posilani casovych zon.\n"); fflush(stdout);
            mysql_free_result(result);
          }       
          mysql_close(&my_connection);
        }
        break;

      case MSG_DB_LOG:
        message_parse(msg, "S", str_log);
        message_free(msg);
        /* pripojeni do DB */     
        mysql_init(&my_connection); 
        if (!mysql_real_connect(&my_connection, DB_SERVER, DB_USER, DB_PASS, DB_NAME, 0, NULL, 0)) {
          fprintf(stderr, "Nepodarilo se pripojit k DB!\n");
          kernel_send_simple_message(sock, MSG_DB_FAIL);
          if (mysql_errno(&my_connection)) {
            fprintf(stderr, "Connection error %d: %s\n",
            mysql_errno(&my_connection), mysql_error(&my_connection));
          }
        } else { 
          /* vytvoreni SQl dotazu a provedeni */
          sprintf(sql_prikaz, "INSERT INTO log ( data ) VALUES ('%s');", str_log);
          if ( mysql_query(&my_connection, sql_prikaz) ) {
            printf("MSG_DB_REQ_ONE_TIME_ZONE: SELECT error: %s\n", mysql_error(&my_connection));
            kernel_send_simple_message(sock, MSG_DB_FAIL);
          } else {
            #if TRACE_MSG
              printf("MSG_DB_LOG: Do DB ulozeno \"%s\"\n", str_log); fflush(stdout);
            #endif
          }
        }
        break;

      case MSG_NONE:
        #if TRACE_MSG
          printf("Jadro nic nechce cekam...\n"); fflush(stdout);
        #endif
        break;

      default:
        printf("Neznama zprava od jadra.\n"); fflush(stdout);
     }
  }
}

/*********************************************/
/*    program vlakna na kontrolu zmeny DB    */
/*********************************************/
static void *program_pejsek() {
  int last_time = 0, now_time;
  MYSQL my_connection;
  MYSQL_RES *result;
  MYSQL_ROW row;
  char sql_prikaz[1000];
  unsigned int num_rows;

  while (1) {
    now_time = time(0);
    if ((now_time-last_time)>TIME_WAIT) {
      last_time = now_time;
      mysql_init(&my_connection); 
      if (!mysql_real_connect(&my_connection, DB_SERVER, DB_USER, DB_PASS, DB_NAME, 0, NULL, 0)) {
        fprintf(stderr, "DB_pejsek: Nepodarilo se pripojit k DB!\n");
        if (mysql_errno(&my_connection)) {
          fprintf(stderr, "DB_pejsek: Connection error %d: %s\n",
          mysql_errno(&my_connection), mysql_error(&my_connection));
        }
      } else { 
        /* vytvoreni SQl dotazu a provedeni */
        sprintf(sql_prikaz, "SELECT renew FROM nastaveni");
        if ( mysql_query(&my_connection, sql_prikaz) ) {
          printf("DB_pejsek: SELECT error: %s\n", mysql_error(&my_connection));
        } else {
          result = mysql_store_result(&my_connection);
          num_rows = mysql_num_rows(result);
          if (num_rows!=0){
            row = mysql_fetch_row(result);
            #if TRACE_MSG
              printf("Pejsek testuji DB...%s\n",row[0]); fflush(stdout);
            #endif
            if (!strcmp(row[0],"1")){
              #if TRACE_MSG
                printf("Pejsek: RENEW.\n"); fflush(stdout);
              #endif
              zamkni();
              zmena_db = 1;
              odemkni();
              sprintf(sql_prikaz, "UPDATE nastaveni SET renew = '0' WHERE renew = '1'");
              if ( mysql_query(&my_connection, sql_prikaz) ) {
                printf("DB_pejsek: UPDATE error: %s\n", mysql_error(&my_connection));
              } else {
                printf("DB_pejsek: RENEW zaregistrovano a zmeneno.\n");
              }
            }
          }
          mysql_free_result(result);
        }
        mysql_close(&my_connection);
      }
    }
  }
  return NULL;
}

/*********************************************/
/*              hlavni program               */
/*********************************************/
int main(void) {  
  int sock, port, ret;
  pthread_t vlakno_pejsek;

  pthread_mutex_init(&zamek, NULL);
  port = d_port;

  /* threads */
  ret = pthread_create(&vlakno_pejsek, NULL, program_pejsek, (void *)(NULL));
  if (ret != 0) printf("Nelze vytvorit vlakno pro pejska: %s\n", strerror(ret));

  if ((sock = module_connect(KERNEL_ADDR, port)) == -1) return 1;
  run(sock);

/*  printf("Press ENTER...\n");
  getchar();*/
  
  close(sock);
//  pthread_join(vlakno_pejsek, NULL);
  
  return 0;
}

