#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>

#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "kernel.h"
#include "def.h"


/* 
 * connect to remote host to given port,
 * return the socket
 */
int module_connect(const char *host, uint16_t port) {
  int sock;
  struct sockaddr_in addr;
  struct hostent *h_info;

  /* create socket */
  if ((sock = socket(AF_INET, SOCK_STREAM, 0))  == -1) {
    printf("Nelze inicializovat soket: %s\n", strerror(errno));
    return -1;
  }

  /* set remote address of kernel */
  if ((h_info = gethostbyname(host)) == NULL) {
    printf("Chyba zjišťování \"%s\": %s\n", host, hstrerror(h_errno));
    goto exception0;
  }
  memset(&addr, 0, sizeof(addr));
  addr.sin_family = AF_INET;
  addr.sin_port = htons(port);
  memcpy(&addr.sin_addr, h_info->h_addr, h_info->h_length);

  /* connect to the kernel */
  if (connect(sock, (struct sockaddr *)&addr, sizeof(addr)) == -1) {
    printf("Chyba při připojování: %s\n", strerror(errno));
    goto exception0;
  }

  return sock;

exception0:
  close(sock);
  return -1;
}


void run(int sock) {
  uint8_t id[8];
//  uint8_t key[16];
  uint8_t bmp[BITMAP_SIZE];
  uint16_t id_sensor;
  message_t *msg;
  int i, pocet_useru;
 
  id_sensor = 0x0112;
				  
  printf("Funkce RUN DB\n"); 
  msg = kernel_recv(sock, 1000, 1);
  if(msg == NULL) {
    fprintf(stdout, "Chyba pri prijmu od kernel: MSG_SN_INIT (%02X).\n", MSG_DB_INIT);
    return;
  }

  message_dump("RUN SN: ", msg);

  if(msg->op != MSG_SN_INIT) {
    message_free(msg);
    printf("Zprava neni MSG_SN_INIT (prijato 0x%02X)\n", MSG_SN_INIT);
    return;
  }
  message_free(msg);
  printf("Prijata zprava od kernel: MSG_SN_INIT (%02X).\n", MSG_SN_INIT);


    printf("posilam MSG_SN_REQUEST_TIME_ZONES..."); fflush(stdout);
    msg = message_create(MSG_SN_REQUEST_TIME_ZONES, "W", id_sensor);
    kernel_send(sock, msg);
    message_free(msg);
    printf("ok [%04x]\n", id_sensor); fflush(stdout);
	  
   // cekam na zpravu co po me bude jadro chtit...
    msg = kernel_recv(sock, 4000, -1);
    if(msg->op != MSG_SN_HEADER) {
      message_free(msg);
      printf("Zprava neni MSG_SN_HEADER (prijato 0x%02X)\n", MSG_SN_HEADER);
      return;
    }
    message_dump("prijata zprava MSG_SN_HEADER: ", msg);
    message_parse(msg, "W", &pocet_useru);
    printf("parsed\n");
  
	for (i=0; i<pocet_useru; i++){
      /* prijem nove zony (s kontrolou) */
      if((msg = kernel_recv(sock, TIMEOUT, -1)) == NULL) return;
      message_dump("prijata zprava MSG_SN_TIM_ZONE: ", msg);
      if(msg->op != MSG_SN_TIME_ZONE || !message_parse(msg, "BT", id, bmp)) {
        printf("sn: prijata chybna zprava MSG_DB_TIME_ZONE (typ %02X, prijato %02X).\n", MSG_SN_TIME_ZONE, msg->op);
        message_free(msg);
        return;
      }
      printf("Prisla mi %d. casova zona.\n",i+1); fflush(stdout);
	}

}

//int main(int argn, char *argc[]) {
int main(void) {  
  int sock;
  int port;

  port = s_port;

  if ((sock = module_connect(KERNEL_ADDR, port)) == -1) return 1;
  run(sock);

  printf("Press ENTER...\n");
  getchar();
  
  close(sock);
  
  return 0;
}
