/* logs.c */

/* Pomocny modul pro vkladani a prochazeni logy. */

#include <malloc.h>
#include <string.h>
#ifdef TEST
#  include <stdio.h>
#endif

#include "kernel.h"


/*
 * inicializace (vynulovani) seznamu logu
 */
void log_init(log_list_t *list) {
  list->logs = NULL;
  list->last = NULL;
  list->current = NULL;
}


/*
 * zrusi seznam zprav
 */
void log_done(log_list_t *list) {
  log_t *log, *next;

  log = list->logs;
  while(log) {
    next = log->next;
    free(log);
    log = next;
  }

  list->logs = NULL;
  list->current = NULL;
  list->last =NULL;
}


/*
 * pridani logu do seznamu
 */
log_t *log_add(log_list_t *list, uint16_t sensor, int type, jis_t user, const char *time, const char *text) {
  log_t *log;

  /* create the item */
  log = malloc(sizeof(log_t));
  log->next = NULL;
  log->sensor = sensor;
  log->type = type;
  if (user) memcpy(log->user, user, sizeof(jis_t));
  else memset(log->user, 0, sizeof(jis_t));
  strncpy(log->time, time, sizeof(log->time) - 1);
  log->time[sizeof(log->time) - 1] = '\0';
  strncpy(log->text, text, sizeof(log->text) - 1);
  log->text[sizeof(log->text) - 1] = '\0';
  
  /* put the item */
  if (!list->logs) {
    list->logs = log;
    list->last = log;
  } else {
    list->last->next = log;
    list->last = log;
  }

  return log;
}


/*
 * vrati prvni zaznam
 */
log_t *log_first(log_list_t *list) {
  list->current = list->logs ? list->logs->next : NULL;
  return list->logs;
}


/*
 * vraci nasledujici zpravy v seznamu
 */
log_t *log_next(log_list_t *list) {
  log_t *log;
  
  if (list->current) {
    log = list->current;
    list->current = list->current->next;
    return log;
  } else return NULL;
}


/*
 * vraci, zda je seznam logu prazdny
 */
int log_isempty(log_list_t *list) {
  return !list || !list->last;
}


#ifdef TEST
int main() {
  log_list_t list;
  log_t *log;
  jis_t user = {1, 2, 3, 4, 5, 6, 7, 8};

  log_init(&list);
  log_add(&list, 1, user, "1.2.2004, 5:6:7", "odmitnuto");
  log_add(&list, 1, user, "1.2.2004, 5:7:0", "prijato");
  log = log_first(&list);
  while(log) {
    printf("typ %d, cas %s, popis %s\n", log->type, log->time, log->text);
    log = log_next(&list);
  }
  log_done(&list);

  return 0;
}
#endif
