/* main.c */

/* Hlavni program jadra. */


#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <pthread.h>
#include <signal.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "kernel.h"
#include "def.h"


static pthread_t d_poslech, s_poslech, m_poslech;
static int d_sock, s_sock, m_sock;
static int kernel_running = 0;  /* informace, ze jadro bezi */


/* ceka na pripojeni,
 * po pripojeni odesle uvodni zpravu,
 * v pripade chyby znovuobnovuje spojeni 
 */
static int main_connect(const char *pfx, int sock_tcp, int initop, const char *initopstr) {
  int sock;
  
  do {
    /* waiting for sensor */
    while (!konec && (sock = kernel_accept(pfx, sock_tcp)) == -1);
    if (konec) return -1;

    /* initial message */  
    llprintf(TRACE_MSG, "%s: sending %s...", pfx, initopstr);
    if (!kernel_send_simple_message(sock, initop)) {
      close(sock);
      sock = -1;
      continue;
    }
    llprintf(TRACE_MSG, "OK\n");
  } while (!konec && sock == -1);
  if (konec) return sock;

  if (!kernel_lock()) goto chyba;
  sock_sn = sock;
  if (!kernel_unlock()) goto chyba;
  
  return sock;
chyba:
  lprintf("%s: (main_connect) kriticka chyba pri pripojovani/odesilani\n", pfx);
  close(sock);
  return -1;
}


static void *vlakno_snimac(void *params) {
  int sock_tcp = *(int *)params, sock;
  int err_cnt;

  lprintf("sn: cekani na DB\n");
  if (!kernel_lock_read()) return NULL;
  if (!kernel_unlock_read()) return NULL;

  lprintf("sn: docekano, socket = %d, cekani na pripojeni\n", sock_tcp);

  if ((sock = main_connect("sn", sock_tcp, MSG_SN_INIT, "MSG_SN_INIT")) == -1) return NULL;
  printf("sn: modul snimacu se pripojil, socket = %d\n", sock);

  /* handle packets, ignore all errors (except oopses) */
  err_cnt = 0;
  while(!konec) {
    switch (sn_recv(sock)) {
      case 0: 
        lprintf("sn: timeout nebo chyba pri zpracovani prikazu\n"); 
        break;
      case 1: 
        lprintf("sn: prikaz zpracovan\n");
        err_cnt = 0;
        break;
      default:
        lprintf("sn: vypadek\n");
        /* vypadek spojeni */
        if (!kernel_lock()) return NULL;
        sock_sn = -1;
        if (!kernel_unlock()) return NULL;
        close(sock);
        if ((sock = main_connect("sn", sock_tcp, MSG_SN_INIT, "MSG_SN_INIT")) == -1) return NULL;
	printf("sn: modul snimace se opet pripojil, socket = %d\n", sock);
        break;
    }
  }

  return NULL;
}


static void *vlakno_databaze(void *params) {
  int sock_tcp = *(int *)params, sock = -1; /* just make gcc happy */
  message_t *msg;
  int op;
  int retval = 0, ok;
  int loadable;  /* lokalni data nahravatelna ==> behem cekani na DB mohou
                  * ostatni moduly pracovat s lokalnimi daty */

  if (!kernel_lock()) return NULL;
  lprintf("db: zamknuto\n");

  /* povoleni startu ostatnich vlaken */
  kernel_running = 1;

  lprintf("db: nactani dat jadra...");
  loadable = kernel_load("kernel-data.txt");
  lprintf("%d\n", loadable);
  /* pri uspesnem nahrani smi ostatni pristupovat k jadru */
  if (loadable) kernel_unlock();

  /* zkousi prvni spojeni s databazi, dokud neni uspech nebo konec */
  lprintf("db: start, socket = %d, cekani na pripojeni...\n", sock_tcp);
  while (!konec && sock_db == -1) {
    /* waiting for database */
    if ((sock = kernel_accept("db", sock_tcp)) == -1) continue;
    printf("db: databaze se pripojila, socket = %d\n", sock);

    /* init into database */
    llprintf(TRACE_MSG, "db: sending MSG_DB_INIT...");
    if (!kernel_send_simple_message(sock, MSG_DB_INIT)) {
      close(sock);
      continue;
    }
    llprintf(TRACE_MSG, "OK (sock = %d)\n", sock);

    /* cte data (opakuje, az se mu to podari, nebo az dojde k fatalni chybe) */
    while (!konec) {
      if (loadable) if (!kernel_lock()) return NULL;
      lprintf("db:1--------------%d, %d\n", sock_db, sock);
      kernel_dbfree();
      lprintf("db:2--------------%d, %d\n", sock_db, sock);
      retval = db_recv(sock);
      lprintf("db:3--------------%d, %d\n", sock_db, sock);
      /* pri nahravatelnosti dat a neuspechu komunikace s databazi nahraje data */
      if (loadable && retval != 1) loadable = kernel_load("kernel-data.txt");
      /* jestli stale nahravatelne, mohou ostatni klidne pokracovat (jinak ne) */
      if (loadable) if (!kernel_unlock()) return NULL;
      /* cekani na dalsi pokus 
       * (pri nenahravatelnosti musi cekat i ostatni - na odemceni jadra) 
       */
      if (retval == 0) usleep(1000 * TIMEOUT);
      else break;
    }
    if (konec || retval == -1) continue;
    
    /* nastaveni soketu databaze - pouzivaji ostatni moduly */
    sock_db = sock;
  }
  lprintf("db: data nahrana (%d, %d)\n", sock_db, sock);
  /* odemceni zamku, museli-li ostatni cekat na spojeni s databazi kvuli tomu,
   * ze neslo nahrat data lokalne
   */
  if (!loadable) if (!kernel_unlock()) return NULL;

  /* poslani MSG_SN_RENEW v pripade, ze uz je aktivni spojeni s cidlama */
  if (!kernel_lock()) return NULL;
  if (sock_sn != -1) sn_invalidate = 1;
  if (!kernel_unlock()) return NULL;

  while (!konec) {
    msg = NULL;
    /* cekani na zpravu nebo fatalni chybu */
    while (!konec && ((msg = kernel_recv(sock, TIMEOUT, 1)) != NULL) && msg->op == MSG_NONE) {
    lprintf("db: -----------cekani (sock = %d, op = %02X)\n", sock, msg->op);
      message_free(msg);
      /* jestlize neni seznam cekajicich logy prazdny, poslou se a presunou se
         do seznamu hotovych logu */
      if (!log_isempty(&pending_logs)) {
        log_t *log;
        char str[STR_MAXLEN];

        lprintf("db: v pending_logs neco je, posle se\n");
        if (!kernel_lock()) return NULL;
        log = log_first(&pending_logs);
        while (log) {
          snprintf(str, STR_MAXLEN, "typ %d, cidlo 0x%04X, JIS %s, cas %s: %s", log->type, log->sensor, id2str(log->user, sizeof(jis_t)), log->time, log->text);
          msg = message_create(MSG_DB_LOG, "S", str);
#ifdef TRACE_MSG
          message_dump("db: log k odeslani: ", msg);
#endif
          if (!kernel_send(sock, msg)) {
            printf("db: zpravy s logem se nepodarilo odeslat, vypadek spojeni\n");
            message_free(msg);
            msg = NULL;
            if (!kernel_unlock()) return NULL;
            goto exc_behind;
	  }
          message_free(msg);
          log_add(&logs, log->sensor, log->type, log->user, log->time, log->text);
          log = log_next(&pending_logs);
        }
        log_done(&pending_logs);
        log_init(&pending_logs);
        if (!kernel_unlock()) return NULL;
      }
    }
    message_dump("db: prijato ", msg);
exc_behind:
    /* pri fatalni chybe se pokusi restartovat spojeni */
    if (!msg) {
      printf("db: spojeni selhalo\n");

      /* ukonceni spojeni */
      if (!kernel_lock()) return NULL;
      close(sock_db);
      sock_db = -1;
      if (!kernel_unlock()) return NULL;

      while (!konec && sock_db == -1) {
        /* cekani na databazi */
        if ((sock = kernel_accept("db", sock_tcp)) == -1) continue;
        /* init into database */
        llprintf(TRACE_MSG, "db: sending again MSG_DB_INIT...");
        if (!kernel_send_simple_message(sock, MSG_DB_INIT)) {
	  close(sock);
	  continue;
	}
        llprintf(TRACE_MSG, "OK\n");

        printf("db: databaze se opet pripojila, socket = %d\n", sock);
        if (!kernel_lock()) return NULL;
        sock_db = sock;
        if (!kernel_unlock()) return NULL;
      }
    
      continue;
    }
    if (konec) return NULL;

    op = msg->op;
    message_free(msg);
    switch(op) {
      case MSG_DB_RENEW:
        lprintf("db: zamykani...");
        if (!kernel_lock()) return NULL;
        lprintf("OK\n");

        printf("db: znovuziskani dat\n");
        kernel_dbfree();

        ok = 0;
        while (!konec && !ok) {
          if (sock_db == -1) {
            /* pokus o znovuspojeni */
            if ((sock = kernel_accept("db", sock_tcp)) == -1) continue;
            /* init into database */
            llprintf(TRACE_MSG, "db: sending again MSG_DB_INIT...");
            if (!kernel_send_simple_message(sock, MSG_DB_INIT)) {
              close(sock);
              continue;
            }
            llprintf(TRACE_MSG, "OK\n");
	    /* uspech */
	    sock_db = sock;
            printf("db: databaze se opet pripojila k ziskani novych dat, socket = %d\n", sock);
          } else {
          /* pokus o ziskani dat */
            while (!konec && (retval = db_recv(sock_db)) == 0) usleep(1000 * TIMEOUT);
            if (retval == 1) ok = 1;
	    else {
            /* zruseni spojeni a opakovat */
              close(sock_db);
              sock_db = -1;
            }
          }
        }
        printf("db: data nahrana\n");

        /* odeslani info cidlu (chyby ignorovany) */
	if (sock_sn != -1) {
//	  kernel_send_simple_message(sock_sn, MSG_SN_RENEW);
          sn_invalidate = 1;
	}

        lprintf("db: odemceni\n");
        if (!kernel_unlock()) return NULL;

        break;
      default:
        printf("db: neocekavana zprava %02X\n", msg->op);
        break;
    }
  }

  return NULL;
}


static void *vlakno_monitor(void *params) {
  int sock_tcp = *(int *)params, sock;
  int err_cnt;

  lprintf("mn: cekani na DB\n");
  if (!kernel_lock_read()) return NULL;
  if (!kernel_unlock_read()) return NULL;

  lprintf("mn: docekano, socket = %d, cekani na pripojeni\n", sock_tcp);

  /* waiting for monitor */
  while (!konec && (sock = kernel_accept("mn", sock_tcp)) == -1);
  if (konec) return NULL;
  printf("mn: pripojen\n");

  /* handle packets, ignore all errors (except oopses) */
  err_cnt = 0;
  while(!konec) {
    switch (mn_recv(sock)) {
      case 1: 
        lprintf("mn: prikaz zpracovan\n");
        err_cnt = 0;
        break;
      case 0: 
        lprintf("mn: timeout nebo chyba pri zpracovani prikazu\n"); 
        err_cnt++;
        break;
      case -1:
        /* vypadek spojeni */
        lprintf("mn: vypadek spojeni\n");
        break;
      default:
        printf("mn, vnitrni chyba\n");
        break;
    }
    lprintf("mn: pozadavek skoncil, ukonceni spojeni\n");
    close(sock);
    while (!konec && (sock = kernel_accept("mn", sock_tcp)) == -1);
    if (konec) return NULL;
  }

  return NULL;
}


static void na_konci() {
  kernel_oops("Konec\n");
  
  /* join all threads */
  pthread_join(d_poslech, NULL);
  pthread_join(s_poslech, NULL);
  pthread_join(m_poslech, NULL);

  /* close connections */
  close(m_sock);
  close(s_sock);
  close(d_sock);

  kernel_done();
}


static void sig_hand(int sig) {
  kernel_oops("main: signal %d, konec\n", sig);
  lprintf("kernel: konec = %d\n", konec);
  
  /* join all threads */
  lprintf("kernel: joining DB thread\n");
  pthread_join(d_poslech, NULL);
  lprintf("kernel: DB thread joined\nkernel: join sensors thread\n");
  pthread_join(s_poslech, NULL);
  lprintf("kernel: sensors thread joined\nkernel: joining monitor\n");
  pthread_join(m_poslech, NULL);
  lprintf("kernel: monitor thread joined\nkernel: all joined\n");
}


int main() {
  struct stat info;
  int ret;

  /* priprava dat jadra */
  kernel_init();

  /* prepare connections */
  if ((d_sock = kernel_listen(d_port)) == -1) goto exception_d;
  if ((s_sock = kernel_listen(s_port)) == -1) goto exception_s;
  if ((m_sock = kernel_listen(m_port)) == -1) goto exception_m;

  /* threads */
  ret = pthread_create(&d_poslech, NULL, vlakno_databaze, &d_sock);
  if (ret != 0) 
    printf("Nelze vytvorit vlakno pro snimac: %s\n", strerror(ret));

  /* mutext must be locked by database, waiting... */
  lprintf("main: cekani na zamknuti DB...");
  while (!kernel_running) usleep(10000);
//  kernel_wait_lock();
  lprintf("OK\n");

  ret = pthread_create(&s_poslech, NULL, vlakno_snimac, &s_sock);
  if (ret != 0) 
    printf("Nelze vytvorit vlakno pro databazi: %s\n", strerror(ret));

  ret = pthread_create(&m_poslech, NULL, vlakno_monitor, &m_sock);
  if (ret != 0) 
    printf("Nelze vytvorit vlakno pro monitor: %s\n", strerror(ret));

  atexit(na_konci);
  signal(SIGINT, sig_hand);
//  signal(SIGTERM, sig_hand);
//  signal(SIGHUP, sig_hand);

  while (!konec) {
    usleep(500 * TIMEOUT);
    if (stat("stop", &info) == 0) konec = 1;
  }
  
  return 0;

/* exceptions */
  close(m_sock);
exception_m:
  close(s_sock);
exception_s:
  close(d_sock);
exception_d:
  return 1;
}

