/* messages.h */

#ifndef MESSAGE_H
#define MESSAGE_H

#include <inttypes.h>
#include <stdarg.h>


/* obšlehnuto z libxine1 */
#if defined(__GNUC__)
#  if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 95)
#    define ATTRIBUTE_PACKED __attribute__ ((packed))
#  else
#    pragma pack(1)
#  endif
#endif


/* ---- zprávy ---- */

/* DB */
#define MSG_MASK_SENSOR 0x80
#define MSG_MASK_MONITOR 0x40
#define MSG_MASK_DB 0x20
#define MSG_MASK_DB 0x20

#define MSG_DB_INIT 0x20
#define MSG_DB_HEADER 0x21
#define MSG_DB_REQUEST_SENSORS 0x22
#define MSG_DB_SENSORS 0x23
#define MSG_DB_REQUEST_USERS 0x24
#define MSG_DB_USER 0x25
#define MSG_DB_REQUEST_TIME_ZONE 0x26
#define MSG_DB_TIME_ZONE 0x27
#define MSG_DB_REQ_ONE_USER 0x28
#define MSG_DB_REQ_ONE_TIME_ZONE 0x2A
#define MSG_DB_RENEW 0x29
#define MSG_DB_REQ_ALL_USERS 0x2C
#define MSG_DB_ALL_USERS 0x2B
#define MSG_DB_FAIL 0x2D
#define MSG_DB_LOG 0x2E

/* SN */
#define MSG_SN_INIT 0x8B                   /*prvni zprava hned po spojeni*/
#define MSG_SN_REQUEST_TIME_ZONES 0x86     /*zadost o TZ*/
#define MSG_SN_REQUEST_USERS 0x84          /*zadost o USERS*/
#define MSG_SN_HEADER 0x81                 /**/
#define MSG_SN_USER 0x85                   /*data s uzivateli*/
#define MSG_SN_TIME_ZONE 0x87              /*data s casovymi zonami*/
#define MSG_SN_REQUEST_1_TIME_ZONE 0x83    /*zadost o 1 casovou zonu*/
#define MSG_SN_REQUEST_1_USER 0x82         /*zadost o 1 uzivatele*/
#define MSG_SN_NOT_FOUND 0x88              /*TZ nebo USER nenalezen*/
#define MSG_SN_ACK 0x89                    /*potvrzovani zprav*/
#define MSG_SN_LOG 0x80                    /*poslani logu od snimace*/
#define MSG_SN_RENEW 0x8A                  /*reset*/
#define MSG_SN_WC 0x8C                     /*cidlo v hajzlu*/
#define MSG_SN_OK 0x8D                     /*cidlo aktivni*/

#define MSG_NONE 0x1F


/* velikost casove zony */
#define BITMAP_SIZE 85

/* maximalni delka retezce (delka retezce ve zprave) */
#define STR_MAXLEN 96

#define SIZEOF_MSG_HEADER (sizeof(uint8_t) + sizeof(uint16_t))


/* úvodní hlavička zprávy */
typedef struct {
  uint8_t op;
  uint16_t size;
  unsigned char data;         /* první bajt dat */
} ATTRIBUTE_PACKED message_t;

typedef uint8_t jis_t[8];


unsigned long hex2ulong(const char *hex);

/*
 * vytvoří rámec k poslání
 */
message_t *message_create(uint8_t op, const char *format, ...);
message_t *message_create_v(uint8_t op, const char *format, va_list ap);

/*
 * vytvoří zprávu s některými daty a s rezervovaným místem 'more_size'
 */
message_t *message_create_part(uint8_t op, uint16_t more_size, 
                               const char *format, ...);
message_t *message_create_part_v(uint8_t op, uint16_t more_size, 
                                const char *format, va_list ap);

/*
 * přidá parametry do dat na pozici 'pos',
 * vyžaduje předem rezervované místo při vytváření zprávy
 */
int message_add_part(message_t *msg, uint16_t pos, const char *format, ...);
int message_add_part_v(message_t *msg, uint16_t pos, const char *format, va_list ap);

/*
 * uvolní vytvořenou zprávu
 */
void message_free(message_t *message);

/*
 * získá data ve zprávě 'message' ve formátu 'format'
 */
int message_parse(message_t *message, const char *format, ...);
int message_parse_v(message_t *message, const char *format, va_list ap);

/*
 * zpracuje část dat formátu 'format' ve zprávě 'message' od pozice 'pos'
 */
int message_parse_part(message_t *message, uint16_t pos, const char *format, ...);
int message_parse_part_v(message_t *message, uint16_t pos, const char *format, va_list ap);

/*
 * testuje konzistentnost zprávy
 */
int message_test(message_t *message, const char* pfx, uint8_t op, const char *format);

/*
 * vypíše rámec na obrazovku
 */
void message_dump(const char *pref, message_t *msg);

/* 
 * prevede pole bytu na hexa-string 
 */
char *id2str(uint8_t *id, size_t n);

/* 
 * prevede hexa-string na id 
 */
void str2id(const char *str, uint8_t *id, size_t n);

/*
int message_db_users_parse(const char *modul, message_t *msg, uint16_t *index, uint16_t max_index, user_t *users);
int message_db_tzs_parse(const char *modul, message_t *msg, uint16_t *index, uint16_t max_index, time_zone_t *tzs);
*/

#else
#  warning messages.h jiz vlozen
#endif
