#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>

#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "kernel.h"
#include "def.h"


/* 
 * connect to remote host to given port,
 * return the socket
 */
int module_connect(const char *host, uint16_t port) {
  int sock;
  struct sockaddr_in addr;
  struct hostent *h_info;

  /* create socket */
  if ((sock = socket(AF_INET, SOCK_STREAM, 0))  == -1) {
    printf("Nelze inicializovat soket: %s\n", strerror(errno));
    return -1;
  }

  /* set remote address of kernel */
  if ((h_info = gethostbyname(host)) == NULL) {
    printf("Chyba zjišťování \"%s\": %s\n", host, hstrerror(h_errno));
    goto exception0;
  }
  memset(&addr, 0, sizeof(addr));
  addr.sin_family = AF_INET;
  addr.sin_port = htons(port);
  memcpy(&addr.sin_addr, h_info->h_addr, h_info->h_length);

  /* connect to the kernel */
  if (connect(sock, (struct sockaddr *)&addr, sizeof(addr)) == -1) {
    printf("Chyba při připojování: %s\n", strerror(errno));
    goto exception0;
  }

  return sock;

exception0:
  close(sock);
  return -1;
}


#ifdef DB
void run(int sock) {
  uint8_t id0[] = {0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08};
  uint8_t id1[] = {0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18};
//  uint8_t id1[] = {0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88};
  uint8_t key0[] = {0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99,
                  0xAA, 0xBB, 0xCC, 0xDD, 0xEE, 0xFF};
//  uint8_t key1[] = {0x10, 0x21, 0x32, 0x43, 0x54, 0x65, 0x76, 0x87, 0x98, 0xA9,
//                  0xBA, 0xCB, 0xDC, 0xED, 0xFE, 0x0F};
  uint8_t bmp[BITMAP_SIZE];
  message_t *msg;
  int op, id_sensor;
  int i;
 
  printf("Funkce RUN DB\n"); 
  msg = kernel_recv(sock, 1000, 1);
  if(msg == NULL) {
    fprintf(stdout, "Chyba pri prijmu od kernel: MSG_DB_INIT (%02X).\n", MSG_DB_INIT);
    return;
  }

  message_dump("RUN DB: ", msg);
  op = msg->op;
  message_free(msg);

  if(op != MSG_DB_INIT) {
    message_free(msg);
    printf("Zprava neni MSG_DB_INIT (prijato 0x%02X)\n", MSG_DB_INIT);
    return;
  }
  printf("Prijata zprava od kernel: MSG_DB_INIT (%02X).\n", MSG_DB_INIT);

  while (1) {
    // cekam na zpravu co po me bude jadro chtit...
    if((msg = kernel_recv(sock, 4000, -1)) == NULL) {
      printf("Chyba pri prijmu od kernel\n");
      return;
    }
    // ziskani nejake zpravy
    message_dump("RUN DB: ", msg);
    if (msg->op == MSG_NONE) {
      printf("Data nebyla prijata\n");
      return;
    }
    switch (msg->op) {
      // chce uzivatele cidla
      case MSG_DB_REQUEST_USERS:
        if (!message_parse(msg, "W", &id_sensor)) return;
        /* dva uzivatele */
        printf("ID_sensor[%d] posilam hlavicku usera...",id_sensor); fflush(stdout);
        message_free(msg);
        msg = message_create(MSG_DB_HEADER, "W", 2);
        kernel_send(sock, msg);
        message_free(msg);
        printf("OK\n"); fflush(stdout);
      
        /* user */
        printf("ID_sensor[%d] posilam 1.usera...",id_sensor); fflush(stdout);
        msg = message_create(MSG_DB_USER, "IBW", id0, 1, 0x0000);
        kernel_send(sock, msg);
        message_dump("Odeslano MSG_DB_USER: ", msg);
        printf("OK\n"); fflush(stdout);

        /* user */
        printf("ID_sensor[%d] posilam 1.usera...",id_sensor); fflush(stdout);
        msg = message_create(MSG_DB_USER, "IBW", id1, 1, 0x0000);
        kernel_send(sock, msg);
        message_dump("Odeslano MSG_DB_USER: ", msg);
        printf("OK\n"); fflush(stdout);
        break;
      // chce cidla
      case MSG_DB_REQUEST_SENSORS:
        printf("posilam cidla..."); fflush(stdout);
        message_free(msg);
        msg = message_create(MSG_DB_SENSORS, "BWWWW", 2, 0x0112, 1, 0x00FF, 2);
        kernel_send(sock, msg);
        printf("OK\n"); fflush(stdout);
        break;
      case MSG_DB_REQUEST_TIME_ZONE:
        if (!message_parse(msg, "W", &id_sensor)) return;

        /* dve zony */
        printf("ID_sensor[%d] posilam hlavicku TZ...",id_sensor); fflush(stdout);
        message_free(msg);
        msg = message_create(MSG_DB_HEADER, "W", 1);
        kernel_send(sock, msg);
        message_free(msg);
        printf("OK\n"); fflush(stdout);
      
        /* zona */
        printf("ID_sensor[%d] posilam 1.TZ...",id_sensor); fflush(stdout);
        for (i = 0; i < BITMAP_SIZE; i++) bmp[i] = i;
        msg = message_create(MSG_DB_TIME_ZONE, "BT", 100, bmp);
        kernel_send(sock, msg);
        printf("OK\n"); fflush(stdout);
        break;

     case MSG_DB_REQ_ALL_USERS:
       printf("Posilam hlavicku pro user_info_t..."); fflush(stdout);
       message_free(msg);
       msg = message_create(MSG_DB_HEADER, "W", 2);
       kernel_send(sock, msg);
       message_free(msg);

       /* info o userovi */
       msg = message_create(MSG_DB_ALL_USERS, "IKKKKWW", id0, key0, "Jan\0\0\0\0\0\0\0\0\0\0\0\0", "Novak\0\0\0\0\0\0\0\0\0\0\0", "A01024\0\0\0\0\0\0\0\0\0", 3, 1);
       if (!kernel_send(sock, msg)) {
	 message_free(msg);
	 return;
       }
       message_free(msg);
       
       printf("OK\n"); fflush(stdout);
       
       /* info o userovi */
       msg = message_create(MSG_DB_ALL_USERS, "IKKKKWW", id1, key0, "Ferda\0\0\0\0\0\0\0\0\0\0", "Mravenec\0\0\0\0\0\0\0\0", "A99138\0\0\0\0\0\0\0\0\0", 1, 2);
       if (!kernel_send(sock, msg)) {
	 message_free(msg);
	 return;
       }
       message_free(msg);
       
       printf("OK\n"); fflush(stdout);
       break;

#if 0       
       /* info o userovi */
       msg = message_create(MSG_DB_ALL_USERS, "IKKKKWW", id1, key0, "Ferda\0\0\0\0\0\0\0\0\0\0", "Mravenec\0\0\0\0\0\0\0\0", "A01024\0\0\0\0\0\0\0\0\0", 1, 2);
       if (!kernel_send(sock, msg)) return;
       
       printf("OK\n"); fflush(stdout);
#endif
       break;
     case MSG_DB_LOG: {
       char str[STR_MAXLEN];

       message_parse(msg, "S", str);
       message_free(msg);
       printf("log: \"%s\"\n", str);
       break;
     }
     default:
       message_dump("neznama zprava: ", msg);
       message_free(msg);
    }
  }
}
#endif


#ifdef SENS
void run(int sock) {
  message_t *msg;
  uint16_t cidlo = 0x0112;
  uint8_t user0[] = {0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08};
  uint8_t user1[] = {0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18};
  uint8_t cas[16];
                  
  uint8_t rezerva = 0xFD; /*Fanda Dvorak :o)*/
 
  
  printf("Funkce RUN SN:\n"); 

  strcpy(cas, "12.1.2003,12:00");

  /*test prijmu MSG_SN_INIT*/
  msg = kernel_recv(sock, 1000, 1);
  if(msg == NULL) {
    printf("modul: chyba pri prijmu od kernel: MSG_SN_INIT (%02X).\n", MSG_SN_INIT);
    return;
  }
  message_free(msg);

  /*poslani logu*/  
  if ((msg = message_create(MSG_SN_LOG, "WIKB", cidlo, user0, cas, rezerva)) == NULL) {
    printf("modul: chyba pri vytvareni logu.\n");
    return;
  }
  if(kernel_send(sock, msg) < 0) {
    printf("modul: chyba pri odesilani logu.\n");
    message_free(msg);
    return;
  }
  message_free(msg);
  
  cas[1] = '3';
 /*poslani logu*/ 
  if ((msg = message_create(MSG_SN_LOG, "WIKB", cidlo, user1, cas, rezerva)) == NULL) {
    printf("modul: chyba pri vytvareni logu.\n");
    return;
  }
  if(kernel_send(sock, msg) < 0) {
    printf("modul: chyba pri odesilani logu.\n");
    message_free(msg);
    return;
  }

  message_free(msg);

  printf("modul: odeslany 2 logy, konec\n");
}
#endif


#ifdef MON
void run(int sock) {

}
#endif


//int main(int argn, char *argc[]) {
int main(void) {  
  int sock;
  int port;

  #ifdef DB
    port = d_port;
  #endif
  #ifdef SENS
    port = s_port;
  #endif
  #ifdef MON
    port = m_port;
  #endif

  if ((sock = module_connect(KERNEL_ADDR, port)) == -1) return 1;
  run(sock);

  printf("Press ENTER...\n");
  getchar();
  
  close(sock);
  
  return 0;
}
