/* saver.c */

/* Ukladani a nahravani dat jadra, ukladani do log souboru. */


#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <malloc.h>

#include "kernel.h"


#define TRACE_SAVER 0

#define MAX_TAG_SIZE 128
#define TAG_FMT "%127s"


typedef struct {
  int type;
  const char *name;
  void *target;
  int used, rule;
} element_t;



static int close_as(FILE *f, const char *name, const char *name_new) {
  /* zavreni */
  if (fclose(f) != 0) {
    printf("kernel: chyba pri zavirani souboru %s: %s\n", name_new, strerror(errno));
    return 0;
  }
  /* smazani stareho a prejmenovani noveho souboru */
  remove(name);
  if (rename(name_new, name) != 0) {
    printf("kernel: chyba pri prejmenovani %s na %s: %s\n", name_new, name, strerror(errno));
    return 0;
  }

  return 1;
}


/*
 * ulozi data jadra
 *
 * predpoklada jadro zamcene pro cteni
 */
int kernel_save(const char *name) {
  int i, j, k;
  FILE *f;
  char *name_new;
  user_t *user;
  time_zone_t *tz;

  lprintf("kernel_save(name=%s): nsensors=%d, nusers=%d\n", name, nsensors, nusers);

  name_new = malloc(strlen(name) + 5);
  sprintf(name_new, "%s.new", name);
  if ((f = fopen(name_new, "w")) == NULL) {
    printf("kernel: chyba pri ukladani: %s\n", strerror(errno));
    goto exception_free;
  }

  fprintf(f, "USERINFOS n %d\n", nusers);
  for (i = 0; i < nusers; i++) {
    llprintf(TRACE_SAVER, "ukladani user info %s %s\n", uzivatele[i].jmeno, uzivatele[i].prijmeni);
    fprintf(f, "  USERINFO ");
    fprintf(f, "jis %s ", id2str(uzivatele[i].id_jis, sizeof(jis_t)));
    fprintf(f, "name1 %s ", uzivatele[i].jmeno);
    fprintf(f, "name2 %s ", uzivatele[i].prijmeni);
    fprintf(f, "pn %s ", uzivatele[i].os_cislo);
    fprintf(f, "logined %d ", uzivatele[i].prihlasen);
    fprintf(f, " in %04X ", uzivatele[i].addr_snimace);
    fprintf(f, "id_user %d id_group %d ", uzivatele[i].id_user, uzivatele[i].id_group);
    fprintf(f, "key %s\n", id2str(uzivatele[i].key, 16));
  }
  fprintf(f, "\n");

  for (i = 0; i < nsensors; i++) {
    fprintf(f, "SNIMAC addr 0x%04X room %d users %d zones %d active %d\n", snimace[i].id, snimace[i].room, snimace[i].nusers, snimace[i].nzones, snimace[i].active);
    for (j = 0; j < snimace[i].nusers; j++) {
      user = snimace[i].users + j;
      llprintf(TRACE_SAVER, "kernel: ukladani info user %p\n", user);
      fprintf(f, "  USER ");
      fprintf(f, "jis %s ", id2str(user->info->id_jis, sizeof(jis_t)));
      fprintf(f, "id_zone %d ", user->id_zone);
      fprintf(f, "rights %04X\n", user->rights);
    }
    for (j = 0; j < snimace[i].nzones; j++) {
      tz = snimace[i].tz + j;
      fprintf(f, "  TZ ");
      fprintf(f, "id %d ", tz->id);
      fprintf(f, "bmp %d", BITMAP_SIZE);
      for (k = 0; k < BITMAP_SIZE; k++) fprintf(f, " 0x%02X", tz->bmp[k]);
      fprintf(f, "\n");
    }
    fprintf(f, "\n");
  }

  if (!close_as(f, name, name_new)) goto exception_free;
  free(name_new);
  
  return 1;
exception_free:
  free(name_new);
  return 0;
}


/* precte ze souboru slovo */
static int read_element(FILE *f, char *buffer) {
  if (fscanf(f, TAG_FMT, buffer) != 1) {
    if (ferror(f)) printf("kernel: chyba cteni ze souboru: %s\n", strerror(errno));
    return 0;
  }

  return 1;
}


static int parse_line_v(FILE *f, char *buffer, const char *fmt, va_list ap) {
  element_t element[20];
  int i, n;

  i = 0;
  if (!fmt) return 0;
  while (fmt[i]) {
    element[i].type = fmt[i] & ~0x80;
    element[i].name = va_arg(ap, const char *);
    element[i].target = va_arg(ap, void *);
    element[i].used = 0;
    element[i].rule = (fmt[i] & 0x80) != 0;
    i++;
  }

  n = strlen(fmt);
  do {
    for (i = 0; i < n; i++) {
      if (strcmp(element[i].name, buffer) == 0) break;
    }
    if (i == n) {
      break;
    }
    element[i].used = 1;
    if (!read_element(f, buffer)) {
      printf("kernel: ocekava se argument elementu '%s'\n", element[i].name);
      return 0;
    }
    switch (element[i].type) {
      case 'b': *((uint8_t *)element[i].target) = hex2ulong(buffer); break;
      case 'w': *((uint16_t *)element[i].target) = hex2ulong(buffer); break;
      case 'i': *((int *)element[i].target) = hex2ulong(buffer); break;
//      case 'H': ((unsigned long *)element[i].target) = hex2ulong(buffer); break;
      case 'J': str2id(buffer, (uint8_t *)element[i].target, sizeof(jis_t)); break;
      case 'K': str2id(buffer, (uint8_t *)element[i].target, 16); break;
      case 'B':  {
         size_t sizebmp, k;
	 uint8_t *bmp = (uint8_t *)element[i].target;

         sizebmp = hex2ulong(buffer);
         for (k = 0; k < sizebmp; k++) {
           if (!read_element(f, buffer)) return 0;
           bmp[k] = hex2ulong(buffer);
         }
         break;
      }
      case 'S': strcpy((char *)element[i].target, buffer); break;
      default:
        printf("kernel, vnitrni chyba: neznamy typ elementu %d\n", element[i].type);
        return 0;	
    }
  } while (read_element(f, buffer));

  for (i = 0; i < n; i++)
    if (!element[i].used) {
//      printf("kernel: chybi element '%s', konec\n", element[i].name);
      return 0;
    }

  return 1;
}


static int parse_line(FILE *f, char *buffer, const char *fmt, ...) {
  va_list ap;
  int retval;

  va_start(ap, fmt);
  retval = parse_line_v(f, buffer, fmt, ap);
  va_end(ap);
  return retval;
}


static int parse_check_line(FILE *f, char *buffer, const char *element, const char *fmt, ...) {
  va_list ap;
  int retval;

  if (strcmp(buffer, element) != 0) {
    printf("kenrel: ocekava se element '%s'\n", element);
    return 0;
  }
  if (!read_element(f, buffer)) {
    printf("kernel: neocekavany konec\n");
    return 0;
  }
  va_start(ap, fmt);
  retval = parse_line_v(f, buffer, fmt, ap);
  va_end(ap);
  return retval;
}


/* nacte data jadra ze souboru 
 *
 * predpoklada jadro zamcene pro zapis
 */
int kernel_load(const char *name) {
  FILE *f;
  char buffer[MAX_TAG_SIZE];
  int uuser, utz;
  int i;

  if ((f = fopen(name, "r")) == NULL) {
    printf("kernel: chyba pri otevirani %s: %s\n", name, strerror(errno));
    return 0;
  }

  if (!read_element(f, buffer)) {
    printf("kernel: prazdny soubor\n");
    return 0;
  }
  if (!parse_check_line(f, buffer, "USERINFOS", "i", "n", &nusers)) goto stop;
  if (nusers > 0) uzivatele = malloc(nusers * sizeof(user_info_t));
  else uzivatele = NULL;
  for (i = 0; i < nusers; i++) {
    user_info_t *info = uzivatele + i;

    if (!parse_check_line(f, buffer, "USERINFO", "JSSSiwiiK", "jis", info->id_jis, "name1", info->jmeno, "name2", info->prijmeni, "pn", info->os_cislo, "logined", &info->prihlasen, "in", &info->addr_snimace, "id_user", &info->id_user, "id_group", &info->id_group, "key", info->key)) goto stop;
  }

  nsensors = 0;
  while (strcmp(buffer, "SNIMAC") == 0) {
    llprintf(TRACE_SAVER, "kernel: %d. snimac\n", nsensors);
    /* cteni info o snimaci */
    if (!read_element(f, buffer)) {
      printf("kernel: ocekavaji se argumenty pro snimac\n");
      return 0;
    }
    if (!parse_line(f, buffer, "wiiii", "addr", &snimace[nsensors].id, "users", &snimace[nsensors].nusers, "zones", &snimace[nsensors].nzones, "room", &snimace[nsensors].room, "active", &snimace[nsensors].active)) goto stop;
    /* ignorovani nekterych polozek */
    snimace[nsensors].active = -1;
    llprintf(TRACE_SAVER, "snimac %04X\n", snimace[nsensors].id);
    
    /* uzivatele */
    if (snimace[nsensors].nusers) snimace[nsensors].users = malloc(snimace[nsensors].nusers * sizeof(user_t));
    else snimace[nsensors].users = NULL;
    for (uuser = 0; uuser < snimace[nsensors].nusers; uuser++) {
      user_t *user = snimace[nsensors].users + uuser;
      user_info_t *ui;
      jis_t jis;
      
      if (!parse_check_line(f, buffer, "USER", "Jii", "jis", jis, "id_zone", &user->id_zone, "rights", &user->rights)) goto stop;
      llprintf(TRACE_SAVER, "  user %s\n", id2str(jis, sizeof(jis_t)));
      if ((ui = kernel_search_user(jis)) == NULL) {
        printf("kernel: uzivatel s JIS %s nenalezen\n", id2str(jis, sizeof(jis_t)));
        goto stop;
      }
      user->info = ui;
    }

    /* casove zony */
    if (snimace[nsensors].nzones) snimace[nsensors].tz = malloc(snimace[nsensors].nzones * sizeof(time_zone_t));
    else snimace[nsensors].tz = NULL;
    for (utz = 0; utz < snimace[nsensors].nzones; utz++) {
      time_zone_t *tz = snimace[nsensors].tz + utz;

      if (!parse_check_line(f, buffer, "TZ", "bB", "id", &tz->id, "bmp", tz->bmp)) goto stop;
      llprintf(TRACE_SAVER, "  tz %d\n", tz->id);
    }
    nsensors++;
  }

stop:
  
  fclose(f);

  return 1;
}


static void kernel_save_loglist(FILE *f, log_list_t *list) {
  log_t *log;
  
  log = log_first(list);
  while (log) {
    fprintf(f, "  %s %s %d %s\n", log->time, id2str(log->user, sizeof(jis_t)), log->type, log->text ? log->text : "");
    log = log_next(list);
  }
}


/*
 * ulozi logy vsech snimacu,
 *
 * predpoklada jadro zamcene pri cteni
 */
int kernel_save_logs(const char *name) {
  FILE *f;
  char *name_new;

  lprintf("kernel_save_log(name=%s): nsensors=%d\n", name, nsensors);

  name_new = malloc(strlen(name) + 5);
  sprintf(name_new, "%s.new", name);

  if ((f = fopen(name_new, "w")) == NULL) {
    printf("kernel: chyba pri vytvareni souboru %s: %s\n", name_new, strerror(errno));
    goto exception_free;
  }

  /* ulozeni logu vsech snimacu */
  fprintf(f, "SENTLOGS\n");
  kernel_save_loglist(f, &logs);
  fprintf(f, "\n");
  fprintf(f, "PENDINGLOGS\n");
  kernel_save_loglist(f, &pending_logs);

  if (!close_as(f, name, name_new)) goto exception_free;
  free(name_new);
  
  return 1;

exception_free:
  free(name_new);
  return 0;
}


#ifdef TEST
int main() {
  int ret, i, j;

  kernel_init();
  printf("load: %d\n", ret = kernel_load("kernel-data.txt"));
  if (ret) {
    printf("nsensors = %d, nusers = %d\n", nsensors, nusers);
    for (i = 0; i < nsensors; i++) {
      printf("sensor %04X: nusers = %d, nzones = %d\n", snimace[i].id, snimace[i].nusers, snimace[i].nzones);
      for (j = 0; j < snimace[i].nusers; j++) printf("  user%d: info = %p\n", j, snimace[i].users[j].info);
      for (j = 0; j < snimace[i].nzones; j++) {
	int k;
	
	printf("  zone%d: id = %d, ", j, snimace[i].tz[j].id);
	for (k = 0; k < BITMAP_SIZE; k++) printf("%02X ", snimace[i].tz[j].bmp[k]);
	printf("\n");
      }
    }
//    printf("save: %d\n", kernel_save("kernel-pokus.txt"));
  }
  kernel_done();

  return 0;
}
#endif
